@@ -4,4 +4,6 @@ from .funcs import *  # noqa: F401, F403
 from .heuristics import *  # noqa: F401, F403
 from .normalize import *  # noqa: F401, F403
 from .progress import *  # noqa: F401, F403
+from .render import *  # noqa: F401, F403
+from .timecodes import *  # noqa: F401, F403
 from .utils import *  # noqa: F401, F403
