@@ -23,6 +23,18 @@ __all__ = [
 
 
 def change_fps(clip: vs.VideoNode, fps: Fraction) -> vs.VideoNode:
+    """
+    Convert the framerate of a clip.
+
+    This is different from AssumeFPS as this will actively adjust
+    the framerate of a clip, not simply set one.
+
+    :param clip:        Input clip.
+    :param fps:         Framerate to conver the clip to. Must be a Fration.
+
+    :return:            Clip with the framerate converted and frames adjusted as necessary.
+    """
+
     src_num, src_den = clip.fps_num, clip.fps_den
     dest_num, dest_den = fps.as_integer_ratio()
 
@@ -47,6 +59,31 @@ def change_fps(clip: vs.VideoNode, fps: Fraction) -> vs.VideoNode:
 def padder(
     clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0, reflect: bool = True
 ) -> vs.VideoNode:
+    """
+    Pad out the pixels on the side by the given amount of pixels.
+
+    There are two padding modes:
+
+        * Filling: This will simply duplicate the row/column *n* amount of pixels.
+        * Reflect: This will reflect the clip for *n* amount of pixels.
+
+    Default mode is `Reflect`. `Filling` can be enabled by setting `reflect=False`.
+
+    Optional Dependencies:
+        * `reflect=False`: `VapourSynth-fillborders <https://github.com/dubhater/vapoursynth-fillborders>`_
+
+    For a 4:2:0 clip, the output must be an even resolution.
+
+    :param clip:        Input clip.
+    :param left:        Padding added to the left side of the clip.
+    :param right:       Padding added to the right side of the clip.
+    :param top:         Padding added to the top side of the clip.
+    :param bottom:      Padding added to the bottom side of the clip.
+    :param reflect:     Whether to reflect the padded pixels.
+                        Default: True.
+    """
+    from ..utils import core
+
     width = clip.width + left + right
     height = clip.height + top + bottom
 
@@ -57,16 +94,16 @@ def padder(
             padder, fmt, 'Values must result in a mod congruent to the clip\'s subsampling ({subsampling})!'
         )
 
-    reflected = vs.core.resize.Point(
-        clip, width, height,
+    reflected = core.resize.Point(
+        clip.std.CopyFrameProps(clip.std.BlankClip()), width, height,
         src_top=-top, src_left=-left,
         src_width=width, src_height=height
-    )
+    ).std.CopyFrameProps(clip)
 
     if reflect:
         return reflected
 
-    return vs.core.fb.FillBorders(
+    return core.fb.FillBorders(  # type: ignore
         reflected, left=left, right=right, top=top, bottom=bottom
     )
 
@@ -78,12 +115,24 @@ FFLOAT = TypeVar('FFLOAT', bound=Callable[..., vs.VideoNode])
 @disallow_variable_format
 @disallow_variable_resolution
 def pick_func_stype(clip: vs.VideoNode, func_int: FINT, func_float: FFLOAT) -> FINT | FFLOAT:
+    """
+    Pick the function matching the sample type of the clip's format.
+
+    :param clip:        Input clip.
+    :param func_int:    Function to run on integer clips.
+    :param func_float:  Function to run on float clips.
+
+    :return:            Function matching the sample type of your clip's format.
+    """
+
     assert clip.format
 
     return func_float if clip.format.sample_type == vs.FLOAT else func_int
 
 
 def set_output(node: vs.RawNode, name: str | bool = True, **kwargs: Any) -> None:
+    """Set output node with optional name, and if available, use vspreview set_output."""
+
     index = len(vs.get_outputs())
 
     ref_id = str(id(node))
@@ -109,8 +158,8 @@ def set_output(node: vs.RawNode, name: str | bool = True, **kwargs: Any) -> None
                 break
 
     try:
-        from vspreview import set_output
-        set_output(node, name, **kwargs)
+        from vspreview import set_output as vsp_set_output
+        vsp_set_output(node, name, **kwargs)
     except ModuleNotFoundError:
         if isinstance(name, str) and isinstance(node, vs.VideoNode):
             node = node.std.SetFrameProp('Name', data=name.title())  # type: ignore
